<?php

namespace Teknisa\DF\Modules\Saida\NFSe;

use Teknisa\DF\Modules\Saida\NFSe\Utilities\Util;
use Teknisa\Libs\Util\Environment;


class NFSeSaoPaulo {
    
    const URL_PRODUCAO = "https://nfews.prefeitura.sp.gov.br/lotenfeasync.asmx?WSDL";
    const URL_PRODUCAO_SYNC= "https://nfe.prefeitura.sp.gov.br/ws/lotenfe.asmx?WSDL";
    
    protected $nomeFuncoes = array(
            "testeenvio"                => "TesteEnvioLoteRPSRequest",
            "testeEnvioLoteRPSAsync"    => "TesteEnvioLoteRPSRequest",
            "envioLoteRPS"              => "EnvioLoteRPSRequest",
            "envioLoteRPSAsync"         => "EnvioLoteRPSRequest",
            "cancelamentoNFe"           => "CancelamentoNFeRequest",
            "ConsultaInformacoesLote"   => "ConsultaInformacoesLoteRequest",
            "consultaNFe"               => "ConsultaNFeRequest",
            "consultaNFeRecebidas"      => "ConsultaNFeRecebidasRequest",
            "consultaNFeEmitidas"       => "ConsultaNFeEmitidasRequest",
            "consultaLote"              => "ConsultaLoteRequest",
            "consultaSituacaoLote"      => "ConsultaSituacaoLoteRequest",
            "envioRPSAsync"             => "EnvioRPSRequest",
            "consultaCNPJ"              => "ConsultaCNPJRequest"
           
    );

    protected $connection;
    protected $environment;
    protected $newCode;
    
    protected $ambiente; //1 = producao, 2 = homologacao
    
    // Dados para identificar o Certificado Digital
    protected $cdempresa;
    protected $cdinscmuni;

    protected $xmlEnviarLoteRps;
    protected $listaRps = array();
    protected $listaAuxRps = array();
    protected $itensRps = array();
    protected $xmlConsultarLoteRps;
    protected $xmlCancelarNotas;
    protected $listaNotasCancelamento = array();

    protected $publicKey;
    protected $privateKeyId;
    protected $caminhoCertificado;
    protected $linkCertificado;
    protected $linkCertKey;
    protected $caminhoXml;

    protected $senhaCertificadoDigital;

    protected $datasEmissao = array();

    protected $CodCidade;// = '7107';
    protected $CPFCNPJRemetente;
    protected $RazaoSocialRemetente;
    protected $ValorTotalServicos;
    protected $ValorTotalDeducoes;
    protected $IdLote;

    protected $IdRPS;
    protected $dadosAssinatura;
    protected $Assinatura;
    protected $NumeroRPS;
    protected $InscricaoMunicipalPrestador;
    protected $TipoRPS;// = 'RPS';
    protected $SerieRPS;// = 'NF';
    protected $DataEmissaoRPS;
    protected $SituacaoRPS;// = 'N';
    protected $SeriePrestacao;// = '99';
    protected $InscricaoMunicipalTomador;
    protected $CPFCNPJTomador;
    protected $RazaoSocialTomador;
    protected $TipoLogradouroTomador;
    protected $LogradouroTomador;
    protected $NumeroEnderecoTomador;
    protected $ComplementoEnderecoTomador;
    protected $TipoBairroTomador;
    protected $BairroTomador;
    protected $CidadeTomador;
    protected $Uf;
    protected $CidadeTomadorDescricao;
    protected $CEPTomador;
    protected $EmailTomador;
    protected $CodigoAtividade;
    protected $AliquotaAtividade;
    protected $TipoRecolhimento;
    protected $MunicipioPrestacao;
    protected $MunicipioPrestacaoDescricao;
    protected $Operacao;
    protected $Tributacao;
    protected $ValorPIS;
	protected $ValorCOFINS;
	protected $ValorINSS;
	protected $ValorIR;
	protected $ValorCSLL;
	protected $ValorIssRetido;
	protected $IssRetido;
	protected $AliquotaPIS;
	protected $AliquotaCOFINS;
	protected $AliquotaINSS;
	protected $AliquotaIR;
	protected $AliquotaCSLL;
	protected $DescricaoRPS;
	protected $DDDPrestador;
	protected $TelefonePrestador;
	protected $DDDTomador;
	protected $TelefoneTomador;

	
	//Inicio Deducoes
	protected $DeducaoPor;
	protected $TipoDeducao;
	protected $CPFCNPJReferencia;
	protected $NumeroNFReferencia;
	protected $ValorTotalReferencia;
	protected $PercentualDeduzir;
	protected $ValorDeduzir;
	//Fim Deducoes

    //Inicio Item
	protected $DiscriminacaoServico;
	protected $Quantidade;
	protected $ValorUnitario;
	protected $ValorTotal;
	protected $Tributavel;
    // Fim Item
    

	public function __construct($connection, Environment $environment, $newCode, $xmlDir, $certUploadDir = null, $certUploadLink = null, $certKeyLink = null) {
        $this->connection = $connection;
        $this->environment = $environment;
        $this->newCode = $newCode;
        
        //$this->setCaminhoXml($xmlDir);
        if($certUploadDir) {
            $this->setCaminhoCertificado($certUploadDir);
        }
        else {
            $this->linkCertificado = $certUploadLink;
            $this->linkCertKey     = $certKeyLink;
        }
	}

	private function setCaminhoXml($xmlDir) {
	    $xmlDir .= "org_". $this->environment->getNrOrg() . DIRECTORY_SEPARATOR;
	    $this->caminhoXml = $xmlDir;
	    if (!is_dir($this->caminhoXml)) {
            mkdir($this->caminhoXml, 0775, true);
            chmod($this->caminhoXml, 0775);
        }
	}

	private function setCaminhoCertificado($certUploadDir) {
	    if (!is_null($certUploadDir)) {
	        $certUploadDir .= "org_". $this->environment->getNrOrg() . DIRECTORY_SEPARATOR;
	        $this->caminhoCertificado = $certUploadDir;
	        if (!is_dir($this->caminhoCertificado)) {
	            mkdir($this->caminhoCertificado, 0775, true);
	            chmod($this->caminhoCertificado, 0775);
	        }
	    }
	}

    private function gerarXmlEnviarLoteRps() {
        $qtdeRps = count($this->listaRps);
        
        sort($this->datasEmissao, SORT_STRING);
        $dataInicio = array_shift($this->datasEmissao);
        $dataFim = array_pop($this->datasEmissao);

        $this->xmlEnviarLoteRps = 
        '<PedidoEnvioLoteRPS xmlns="http://www.prefeitura.sp.gov.br/nfe">'
            .'<Cabecalho xmlns="" Versao="1">'
                .'<CPFCNPJRemetente>';
                	if (strlen($this->CPFCNPJRemetente) == 11) {
    					$this->xmlEnviarLoteRps .='<CPF>'.$this->CPFCNPJRemetente.'<CPF>';
    				}
    				else {
    					$this->xmlEnviarLoteRps .= '<CNPJ>'.$this->CPFCNPJRemetente.'</CNPJ>';
    				}
    				$this->xmlEnviarLoteRps .=  '</CPFCNPJRemetente>'
                .'<transacao>true</transacao>'
                .'<dtInicio>'.$dataInicio.'</dtInicio>'
                .'<dtFim>'.$dataFim.'</dtFim>'
                .'<QtdRPS>'.$qtdeRps.'</QtdRPS>'
                .'<ValorTotalServicos>'.$this->ValorTotalServicos.'</ValorTotalServicos>'
                .'<ValorTotalDeducoes>'.$this->ValorTotalDeducoes.'</ValorTotalDeducoes>'
            .'</Cabecalho>';
           
                foreach ($this->listaRps as $rps) {
                $this->xmlEnviarLoteRps .= '<RPS xmlns="">';
                $this->xmlEnviarLoteRps .= $rps;
                $this->xmlEnviarLoteRps .= '</RPS>';
                }
                $this->xmlEnviarLoteRps .= '</PedidoEnvioLoteRPS>';
 
    }
    
	private function adicionarRpsEnviarLote() {
        $xmlRps = "";
        foreach ($this->itensRps as $item) {
            $xmlRps .= '<Assinatura>'.$this->Assinatura.'</Assinatura>'.
        		'<ChaveRPS>'
        			.'<InscricaoPrestador>'.$this->InscricaoMunicipalPrestador.'</InscricaoPrestador>'
        			.'<SerieRPS>'.$this->SerieRPS.'</SerieRPS>'
        			.'<NumeroRPS>'.$this->NumeroRPS.'</NumeroRPS>'
        		.'</ChaveRPS>'
            	   .'<TipoRPS>'.$this->TipoRPS.'</TipoRPS>'
            		.'<DataEmissao>'.substr($this->DataEmissaoRPS, 0, 10).'</DataEmissao>'
            		.'<StatusRPS>'.$this->SituacaoRPS.'</StatusRPS>'
            		.'<TributacaoRPS>'.$this->Tributacao.'</TributacaoRPS>'
                    .'<ValorServicos>'.$item["valorServico"].'</ValorServicos>'
                    .'<ValorDeducoes>'.$item["valorDeducao"].'</ValorDeducoes>'
                    .'<ValorPIS>'.$this->ValorPIS.'</ValorPIS>'
    				.'<ValorCOFINS>'.$this->ValorCOFINS.'</ValorCOFINS>'
    				.'<ValorINSS>'.$this->ValorINSS.'</ValorINSS>'
    				.'<ValorIR>'.$this->ValorIR.'</ValorIR>'
    				.'<ValorCSLL>'.$this->ValorCSLL.'</ValorCSLL>'
                    .'<CodigoServico>'.$this->CodigoAtividade.'</CodigoServico>'
                    .'<AliquotaServicos>'.$this->AliquotaAtividade.'</AliquotaServicos>'
                    .'<ISSRetido>'.$this->IssRetido.'</ISSRetido>'
                    .'<CPFCNPJTomador>'
                    .'<CNPJ>'.$this->CPFCNPJTomador.'</CNPJ>'
                    .'</CPFCNPJTomador>';
                    if($this->InscricaoMunicipalTomador != 'ISENTA' && $this->InscricaoMunicipalTomador != 'ISENTO'){
                        $xmlRps .='<InscricaoMunicipalTomador>'.$this->InscricaoMunicipalTomador.'</InscricaoMunicipalTomador>';
                    }
                      if($this->InscricaoEstadualTomador != 'ISENTA' && $this->InscricaoEstadualTomador != 'ISENTO'){
                        $xmlRps .='<InscricaoEstadualTomador>'.$this->InscricaoEstadualTomador.'</InscricaoEstadualTomador>';
                    }
                      $xmlRps .='<RazaoSocialTomador>'.$this->RazaoSocialTomador.'</RazaoSocialTomador>'
                    .'<EnderecoTomador>'
                        .'<TipoLogradouro>'.$this->TipoLogradouroTomador.'</TipoLogradouro>'
                        .'<Logradouro>'.$this->LogradouroTomador.'</Logradouro>'
                        .'<NumeroEndereco>'.$this->NumeroEnderecoTomador.'</NumeroEndereco>'
                        .'<ComplementoEndereco>'.$this->ComplementoEnderecoTomador.'</ComplementoEndereco>'
                        .'<Bairro>'.$this->BairroTomador.'</Bairro>'
                        .'<Cidade>'.$this->CidadeTomador.'</Cidade>'
                        .'<UF>'.$this->Uf.'</UF>'
                        .'<CEP>'.$this->CEPTomador.'</CEP>'
                    .'</EnderecoTomador>'
                    .'<EmailTomador>'.$this->EmailTomador.'</EmailTomador>'
                    .'<Discriminacao>'.$item["DiscriminacaoServico"].'</Discriminacao>'
                    .'<ValorCargaTributaria>'.$this->CargaTributaria.'</ValorCargaTributaria>';
                    if($this->Tributacao != "T") { //  Não  informe o município onde o serviço foi prestado para serviço tributado em São Paulo ou exportação de serviços
                        $xmlRps .='<MunicipioPrestacao>'.$this->MunicipioPrestacao.'</MunicipioPrestacao>';
                    }
        }
        $this->listaRps[] = $xmlRps;
        
	}

    private function getXmlAssinadoEnviarLoteRps() {
        $this->xmlEnviarLoteRps = str_replace("&", "&amp;", $this->xmlEnviarLoteRps);
        $this->substituiCaracteresEspeciais();
        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($this->xmlEnviarLoteRps, LIBXML_NOBLANKS);
        $domXml = $this->removerTagsVazias($domXml);
        $domXml->formatOutput = false;
        $xmlTemp = $domXml->saveXML();
        
        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xmlTemp);
        $domXml->encoding = "UTF-8";
        $domXml = $this->assinaXml($domXml, array("PedidoEnvioLoteRPS"));
        if ($domXml === false) {
            throw new \Exception("Certificado digital não encontrado!");
        }
        else {
            $this->limparListaRps();
           return $domXml->saveXML($domXml->documentElement);
        }
    }
    
    private function substituiCaracteresEspeciais() {
        $this->xmlEnviarLoteRps = str_replace("á", "a", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("â", "a", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("é", "e", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("í", "i", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("ó", "o", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("ú", "u", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("ç", "c", $this->xmlEnviarLoteRps);
        
        $this->xmlEnviarLoteRps = str_replace("Á", "A", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Â", "A", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("É", "E", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Í", "I", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Ó", "O", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Ú", "U", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Ç", "C", $this->xmlEnviarLoteRps);
        
        $this->xmlEnviarLoteRps = str_replace("ã", "a", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("à", "a", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("ê", "e", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("ó", "o", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("ô", "o", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("ü", "u", $this->xmlEnviarLoteRps);
        
        $this->xmlEnviarLoteRps = str_replace("Ã", "A", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("À", "A", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Ê", "E", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Ó", "O", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Ô", "O", $this->xmlEnviarLoteRps);
        $this->xmlEnviarLoteRps = str_replace("Ü", "U", $this->xmlEnviarLoteRps);
        
    }
    
    private function removerTagsVazias(\DOMDocument $xml) {
        $xpath = new \DOMXPath($xml);
        foreach($xpath->query('//*[not(node())]') as $node) {
            if (!$node->hasAttributes()) {
                $node->parentNode->removeChild($node);
            }
        }
        return $xml;
    }

    private function limparListaRps() {
        $this->listaRps = array();
        $this->datasEmissao = array();
    }

    public function consultarLoteRps($lote) {
        $this->ambiente = Util::getAmbiente($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCMUNI"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;
        $inscricaoJuridica = str_replace(array("-", "/", ".", " "), "", $lote["NRINSJURIEST"]);
    
        $Situacao = $this->consultaSituacaoLote($inscricaoJuridica, $lote["NRPROTOCOLORPS"] , $lote);

        if(is_array($Situacao)){
            return $Situacao;
        }else {
            $NumeroLote = $Situacao;
        }


        $this->cdempresa = $lote["CDEMPLOTERPS"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $lote["CDINSCMUNI"]);
        
   
        
        //$this->RazaoSocialRemetente = $lote["NMRAZSOCEMPR"];
        $this->gerarXmlConsultarLoteRps($inscricaoJuridica, $NumeroLote);
        $xml = $this->getXmlAssinadoConsultaLoteRps($lote["CDEMPLOTERPS"], $lote["CDINSCMUNI"]);
        $resultado = $this->consultarLoteRpsWS($xml, $lote["CDEMPLOTERPS"], $lote["NRINSJURIEST"], $lote["NRSEQLOTERPS"]);
        
        if (is_string($resultado)) {
            $domResult = new \DOMDocument("1.0", "UTF-8");
            $domResult->loadXML($resultado);
            $sucesso = $domResult->getElementsByTagName("Sucesso");
            
            if($sucesso->length > 0) {
                if ($domResult->getElementsByTagName("Sucesso")->item(0)->nodeValue == "true") {
                    
    
                    $alertas = $domResult->getElementsByTagName("Erro");
                    if ($alertas->length > 0) {
    
                        $erros = array();
                        foreach ($alertas as $alerta) {
                            $erros[] = array(
                                "CODIGO" => Util::getChildNodeByName($alerta, "Codigo")->nodeValue,
                                "DESCRICAO" => Util::getChildNodeByName($alerta, "Descricao")->nodeValue
                            );
                        }
                        return $erros;
                    }
                    else {
        
                        $nodeList = $domResult->getElementsByTagName("NFe");
                        for ($i = 0; $i < $nodeList->length; $i++) {
                            $chaveNfe = Util::getChildNodeByName($nodeList->item($i), "ChaveNFe");
                            $numero = Util::getChildNodeByName($chaveNfe, "NumeroNFe")->nodeValue;
                            $numeroNota = str_pad($numero, 9, "0", STR_PAD_LEFT);
                            $codigoVerificacao = substr(Util::getChildNodeByName($chaveNfe, "CodigoVerificacao")->nodeValue, 0, 8);
                            $chaveRps = Util::getChildNodeByName($nodeList->item($i), "ChaveRPS");
                            $serieRps = Util::getChildNodeByName($chaveRps, "SerieRPS")->nodeValue;
                            $numeroRps = Util::getChildNodeByName($chaveRps, "NumeroRPS")->nodeValue;
                            $numeroRps = str_pad($numeroRps, 15, "0", STR_PAD_LEFT);
                            $dataEmissaoRPS = Util::getChildNodeByName($nodeList->item($i), "DataEmissaoRPS")->nodeValue;
                            $dataEmissaoRPS = date_create_from_format("Y-m-d", $dataEmissaoRPS);
                            $dataEmissaoRPS = $dataEmissaoRPS->format("d/m/Y");

                            Util::atualizarNumeroRps($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], $numeroNota, $codigoVerificacao, 4, $dataEmissaoRPS, $dataEmissaoRPS, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                            
                            Util::atualizarNotaConsulta($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], $numeroNota, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                            
                            Util::inserirArquivoXml($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], "A", $resultado, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                        }
                        
                        Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 4, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                        
                        return true;
                    }
                }
                else {
                    $chave = "NUMERORPS".$lote["CDEMPLOTERPS"].$lote["NRINSJURIEST"].$lote["CDSERIERPS"];
                    $nrRps = $this->newCode->getCodeByTableName($chave, $this->environment->getNrOrg(), 1, 15);
                    $qtdRps = $lote["QTDERPS"];
                    $nrRps = $nrRps - ($qtdRps + 1);
                    
                    Util::atualizarNovoCodigo($this->connection, $lote["CDEMPLOTERPS"], $lote["NRINSJURIEST"], $lote["CDSERIERPS"],  $nrRps, $this->environment->getNrOrg());
                    
                    Util::atualizarSituacaoNumeroRpsPeloLote($this->connection, $lote["CDSERIERPS"], $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                    
                    Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
                    $erros = array();
                    $nodeList = $domResult->getElementsByTagName("Erro");
                    foreach ($nodeList as $erro) {
                        $erros[] = array(
                            "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                            "DESCRICAO" => Util::getChildNodeByName($erro, "Descricao")->nodeValue
                        );
                    }
                    return $erros;
                }
            }
            else {
                $chave = "NUMERORPS".$lote["CDEMPLOTERPS"].$lote["NRINSJURIEST"].$lote["CDSERIERPS"];
                $nrRps = $this->newCode->getCodeByTableName($chave, $this->environment->getNrOrg(), 1, 15);
                $qtdRps = $lote["QTDERPS"];
                
                $nrRps = $nrRps - ($qtdRps + 1);
                    
                Util::atualizarNovoCodigo($this->connection, $lote["CDEMPLOTERPS"], $lote["NRINSJURIEST"], $lote["CDSERIERPS"],  $nrRps, $this->environment->getNrOrg());
                    
                Util::atualizarSituacaoNumeroRpsPeloLote($this->connection, $lote["CDSERIERPS"], $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                
                Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());

                $erros = array();
                $nodeList = $domResult->getElementsByTagName("Erro");
                foreach ($nodeList as $erro) {
                    $erros[] = array(
                        "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                        "DESCRICAO" => Util::getChildNodeByName($erro, "Descricao")->nodeValue
                    );
                }
                return $erros;
            }
        }
        else {
            return false;
        }
    }

    private function consultarLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        $nomeFuncao = "consultaLote";
        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        //file_put_contents($this->caminhoXml . $prefixoArquivo."env.xml", $xml);
        $retorno = $this->enviarParaWebservice($nomeFuncao, $xml);
        //file_put_contents($this->caminhoXml . $prefixoArquivo."resp.xml", $retorno);
        return $retorno;
    }
    
    private function enviarParaWebservice($nomeFuncao, $conteudo) {
        if($this->caminhoCertificado) {
            $caminho = $this->caminhoCertificado . $this->cdempresa . "_" . $this->cdinscmuni . DIRECTORY_SEPARATOR . "certKey.pem";
            $unlink  = false;
        }
        else {
            $certKey = $this->linkCertKey;
            $hash    = uniqid();
            if (is_dir("/tmp")) {
                $caminho= "/tmp/certKey$hash.pem";
            }
            else {
                $caminho = __DIR__ . DIRECTORY_SEPARATOR ."certKey$hash.pem";
            }
            file_put_contents($caminho, file_get_contents($certKey));
            $unlink = true;
        }
       $conteudo = str_replace( "<", "&lt;", $conteudo);
       $conteudo = str_replace( ">", "&gt;", $conteudo);
       if($nomeFuncao === 'cancelamentoNFe' || $nomeFuncao === 'envioLoteRPS' || $nomeFuncao === 'consultaLote'){
            $lotUri = self::URL_PRODUCAO_SYNC;     
        }else {
            $lotUri = self::URL_PRODUCAO;    
        }
       $conteudo = $this->createSoapText($conteudo, $nomeFuncao);
       $soapVar = new \SoapVar($conteudo, XSD_ANYXML);
      
            $resultado = $this->buildSoapCall(
            $caminho,
            $lotUri,
            $lotUri,
            $nomeFuncao,
            $soapVar,
            $conteudo
        );
  
        if($unlink) unlink($caminho);
      
        return $resultado->outputXML;
    }
    
    public function createSoapText($conteudo, $nomeFuncao) {
        $tagRequest = $this->nomeFuncoes[$nomeFuncao];
       
        $mensagemSoap= '<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:nfe="http://www.prefeitura.sp.gov.br/nfe">'
            .'<soapenv:Header/>'
            	 .'<soapenv:Body>'
            		  .'<nfe:'.$tagRequest.'>';
            		     if($nomeFuncao === 'cancelamentoNFe' || $nomeFuncao === 'envioLoteRPS' || $nomeFuncao === 'consultaLote'){
            		        $mensagemSoap .= '<nfe:VersaoSchema>1</nfe:VersaoSchema>';
            		     }else{
            		         $mensagemSoap .= '<nfe:versaoSchema>1</nfe:versaoSchema>';
            		     }
            		$mensagemSoap .= '<nfe:MensagemXML>
            			 '.$conteudo.'</nfe:MensagemXML>'
            	     .'</nfe:'.$tagRequest.'>'
            	 .'</soapenv:Body>'
         .'</soapenv:Envelope>';
        
        return $mensagemSoap;
    }

    public function buildSoapCall($cert, $location, $wsdl, $functionName, $soapVar, $xmlLote) {
        try {
            $msgSize = strlen($xmlLote);
            $parameters = [
                "Content-Type: text/xml;charset=UTF-8",
                "SOAPAction: \"http://www.prefeitura.sp.gov.br/nfe/ws/$functionName\"",
                "Content-length: $msgSize"
            ];
            $oCurl = curl_init();
            curl_setopt($oCurl, CURLOPT_URL, $wsdl);
            curl_setopt($oCurl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
            curl_setopt($oCurl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1); // como a URL é um web Service, essa linha serve par ajustar a versão da setagem do url 
            curl_setopt($oCurl, CURLOPT_HEADER, 1);
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($oCurl, CURLOPT_SSLVERSION, 0);
            curl_setopt($oCurl, CURLOPT_SSLCERT, $cert);
            curl_setopt($oCurl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($oCurl, CURLOPT_POST, true);
            curl_setopt($oCurl, CURLOPT_POSTFIELDS, $xmlLote);
            curl_setopt($oCurl, CURLOPT_HTTPHEADER, $parameters);
            
            $response = curl_exec($oCurl);
            $this->soaperror = curl_error($oCurl);
            $ainfo = curl_getinfo($oCurl);
            if (is_array($ainfo)) {
                $this->soapinfo = $ainfo;
            }
            $headsize = curl_getinfo($oCurl, CURLINFO_HEADER_SIZE);
            $httpcode = curl_getinfo($oCurl, CURLINFO_HTTP_CODE);
            curl_close($oCurl);
            $responseHead = trim(substr($response, 0, $headsize));
            $responseBody = trim(substr($response, $headsize));
            if ($httpcode == 404) {
                throw new \Exception('Erro 404 ao comunicar com a Sefaz.', 1);
            } else if ($httpcode == 500) {
                throw new \Exception($responseBody, 1);
            } else if ($httpcode != 200) {
                throw new \Exception('Erro inesperado.', 1);
            } else {
                if ($wsdl == self::URL_PRODUCAO_SYNC){
                    $inicioResponse = strpos($responseBody, "&lt;");
                    $length         = strrpos($responseBody, "&gt;") - $inicioResponse + 4;
                    $responseBody   = substr($responseBody, $inicioResponse, $length);
                    $responseBody   = str_replace("&lt;", "<", $responseBody);
                    $responseBody   = str_replace("&gt;", ">", $responseBody);
                    $response       = new \stdClass();
                    $response->outputXML = $responseBody;
                    return $response;
                }else{
                    $inicioResponse = strpos($responseBody, "<RetornoXML>");
                    $length         = strrpos($responseBody, "</RetornoXML>") - $inicioResponse + 13;
                    $responseBody   = substr($responseBody, $inicioResponse, $length);
                    $responseBody   = str_replace("&lt;", "<", $responseBody);
                    $responseBody   = str_replace("&gt;", ">", $responseBody);
                    $response       = new \stdClass();
                    $response->outputXML = $responseBody;
                    return $response;
                }
            }

        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function cancelarNotas($notas, $cdmotivo, $motivo, $cdmunicibge, $cdmunicsiafi) {
        $this->cdempresa = $notas[0]["CDEMPRESA"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $notas[0]["CDINSCMUNI"]);
        $this->ambiente = Util::getAmbiente($this->connection, $notas[0]["CDEMPRESA"], $notas[0]["CDINSCMUNI"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;

        $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $notas[0]["CDEMPRESA"], $notas[0]["CDINSCMUNI"], $this->environment->getNrOrg());
        
        //$this->RazaoSocialRemetente = $notas[0]["NMRAZSOCEMPR"];
        $this->CPFCNPJRemetente = $notas[0]["NRINSJURIEST"];
        foreach ($notas as $nota) {
            $cdinscmuni =  str_replace(array("/","-","."), "", $nota["CDINSCMUNI"]);
            
            $assinatura = 
                str_pad($cdinscmuni, 8, "0", STR_PAD_LEFT)
                .str_pad($nota["NRNOTAFISC"], 12, "0", STR_PAD_LEFT);
                
            try{
                $data = file_get_contents($this->linkCertificado);
            }
            catch(\Exception $e) {
                return false;
            }
            
            if(empty($data)) return false;
            openssl_pkcs12_read($data, $cert, $this->senhaCertificadoDigital);
                
            $this->publicKey = $this->getPublicKey($cert["cert"]);
            $this->privateKeyId = openssl_get_privatekey($cert["pkey"]);

            $dados = $assinatura;                                                     
            $assinatura = '';
            
            openssl_sign($dados, $assinatura, $this->privateKeyId, OPENSSL_ALGO_SHA1);
            openssl_free_key($this->privateKeyId); // "Libera/esvazia" a chave pra ser usada novamente
            $signatureValue = base64_encode($assinatura);
        
            $this->adicionarNotaCancelamento($nota["NRNOTAFISC"], $cdinscmuni, $signatureValue);
        }
        
        $this->gerarXmlCancelarNotas($cdmunicsiafi);
        $xml = $this->getXmlAssinadoCancelarNotas();
        $resultado = $this->cancelarNotasWS($xml, $notas[0]["CDEMPRESA"], $notas[0]["NRINSJURIEST"]);
        
        if (is_string($resultado)) {
            $domResult = new \DOMDocument("1.0", "UTF-8");
            $domResult->loadXML($resultado);
            if ($domResult->getElementsByTagName("Sucesso")->item(0)->nodeValue == "true") {
                foreach ($notas as $nota) {
                    Util::inserirNotaCancelada($this->connection, "S", "C", $motivo, "C", $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);
                    Util::atualizarNotaCancelada($this->connection, "N", $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg());
                }
                return true;
            }
            else {
                $erros = array();
                $nodeList = $domResult->getElementsByTagName("Erro");
                foreach ($nodeList as $erro) {
                    $erros[] = array(
                        "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                        "DESCRICAO" => Util::getChildNodeByName($erro, "Descricao")->nodeValue
                    );
                }
                return $erros;
            }
        }
        else {
            return false;
        }
    }

    private function adicionarNotaCancelamento($nrnotafisc, $cdinscmuni,  $assinatura) {
        $xml = '<Detalhe xmlns="">'
        .'<ChaveNFe>'
            .'<InscricaoPrestador>'.$cdinscmuni.'</InscricaoPrestador>'
            .'<NumeroNFe>'.$nrnotafisc.'</NumeroNFe>'
        .'</ChaveNFe>'
        .'<AssinaturaCancelamento>'.$assinatura.'</AssinaturaCancelamento>'
        .'</Detalhe>';
        
         $this->listaNotasCancelamento[] = $xml;
    }

    private function gerarXmlCancelarNotas($cdmunicsiafi) {
        $this->xmlCancelarNotas = '
        <PedidoCancelamentoNFe xmlns="http://www.prefeitura.sp.gov.br/nfe">'
        .'<Cabecalho xmlns="" Versao="1">'
                .'<CPFCNPJRemetente>';
                	if (strlen($this->CPFCNPJRemetente) == 11) {
    					$this->xmlCancelarNotas .='<CPF>'.$this->CPFCNPJRemetente.'<CPF>';
    				}
    				else {
    					$this->xmlCancelarNotas .= '<CNPJ>'.$this->CPFCNPJRemetente.'</CNPJ>';
    				}
    				$this->xmlCancelarNotas .=  '</CPFCNPJRemetente>'
                .'<transacao>true</transacao>'
        .'</Cabecalho>';
                foreach ($this->listaNotasCancelamento as $nota) {
                    $this->xmlCancelarNotas .= "\n" . $nota;
                }
            $this->xmlCancelarNotas .= '</PedidoCancelamentoNFe>';
    }

    private function getXmlAssinadoCancelarNotas() {
        $this->xmlCancelarNotas = str_replace("&", "&amp;", $this->xmlCancelarNotas);

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($this->xmlCancelarNotas, LIBXML_NOBLANKS);
        $domXml->formatOutput = false;
        $xmlTemp = $domXml->saveXML();

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xmlTemp);
        $domXml->encoding = "UTF-8";
        $domXml = $this->assinaXml($domXml, array("PedidoCancelamentoNFe"));
        if ($domXml === false) {
            throw new \Exception("Certificado digital não encontrado!");
        }
        else {
            $this->listaNotasCancelamento = array();
            return $domXml->saveXML($domXml->documentElement);
        }
    }

    private function cancelarNotasWS($xml, $cdempresa, $inscricao) {
        $nomeFuncao = "cancelamentoNFe";
        $dataAtual = date_create();
        $dataAtual = $dataAtual->format("Y-m-d_H-i-s");
        $prefixoArquivo = $cdempresa.$inscricao.$dataAtual."_".$nomeFuncao."_";
        //file_put_contents($this->caminhoXml . $prefixoArquivo."env.xml", $xml);
        $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        //file_put_contents($this->caminhoXml . $prefixoArquivo."resp.xml", $resultado);
        return $resultado;
    }

    private function gerarXmlConsultarLoteRps($inscricao,$NumeroLote) {
        $this->xmlConsultarLoteRps = '<PedidoConsultaLote xmlns="http://www.prefeitura.sp.gov.br/nfe">'
        .'<Cabecalho xmlns="" Versao="1">'
            .'<CPFCNPJRemetente>';
              	if (strlen($inscricao) == 11) {
    					$this->xmlConsultarLoteRps .='<CPF>'.$inscricao.'<CPF>';
    				}
    				else {
    					$this->xmlConsultarLoteRps .= '<CNPJ>'.$inscricao.'</CNPJ>';
    				}
    				$this->xmlConsultarLoteRps .='</CPFCNPJRemetente>'
            .'<NumeroLote>'.$NumeroLote.'</NumeroLote>'
        .'</Cabecalho>'
         .'</PedidoConsultaLote>';
    }

    public function enviarLoteRps($notas) {
        $erros = array("TIPO" => 1);
        $this->listaAuxRps = array();

        $nrlanctonf = $notas[0]["NRLANCTONF"];
        $cdfilial = $notas[0]["CDFILIAL"];

        $notafiscal = Util::getCorpoNF($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
        
        $this->cdempresa = $notafiscal["CDEMPRESA"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $notafiscal["CDINSCMUNIEMP"]);
        
        $this->ambiente = Util::getAmbiente($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;

        $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());
        
        $numeroLote = Util::inserirLoteRps($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCESTA"], $notafiscal["CDINSCMUNIEMP"], $notafiscal["NRINSJURIEST"], $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);
        
        $this->NumeroLote = $numeroLote;
        $this->CodCidade = "7107";
        $this->IdLote = "LOTE_".$notafiscal["CDEMPRESA"]."_".$numeroLote;
        $this->CPFCNPJRemetente = str_replace(array("/", "-", ".", " "), "", $notafiscal["NRINSJURIEST"]);
        $this->InscricaoMunicipalPrestador = str_replace(array("/", "-", ".", " "), "", $notafiscal["CDINSCMUNIEMP"]);//001897870
        $this->InscricaoEstadualTomador = str_replace(array(".","-","/"), "", $notafiscal["NRINSESTCLIE"]);
        $this->RazaoSocialRemetente = $notafiscal["NMRAZSOCEMIT"];

        $this->ValorTotalServicos = 0.0;
        $this->ValorTotalDeducoes = 0.0;

        foreach ($notas as $nota) {
            $nrlanctonf = $nota["NRLANCTONF"];
            $cdfilial = $nota["CDFILIAL"];
            $valorDeducao = ((float)$notafiscal["VRNOTAFISC"] - (float)$notafiscal["VRPRODUTOS"]);
            $notafiscal = Util::getCorpoNF($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

	        if (empty($notafiscal["CDINSCMUNIEMP"])) {
	            $erros[] = array(
	                "CODIGO" => $notafiscal["NRLANCTONF"],
	                "DESCRICAO" => "Inscrição municipal deve ser relacionada com a empresa informada."
	            );
	        }
            $this->ValorTotalServicos += (float)$notafiscal["VRPRODUTOS"];
        	$this->ValorTotalDeducoes += ((float)$notafiscal["VRNOTAFISC"] - (float)$notafiscal["VRPRODUTOS"]);
            
            $this->SerieRPS = 'NF';
            
            if (empty($this->NumeroRPS)) {
                $this->NumeroRPS = (int)Util::getProximoCodigoNumeroRps($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["NRINSJURIEST"], $notafiscal["CDSERIERPS"], $this->environment->getNrOrg());
            }
            else {
                $this->NumeroRPS += 1;
            }

            $this->IdRPS = "RPS_" . $this->SerieRPS . "_" . $this->NumeroRPS;
            $this->TipoRPS = "RPS";
            $this->SituacaoRPS = "N";
            $this->SerieRPS = $notafiscal["CDSERIERPS"];

            $dataAtual = date_create();
            
            if(!isset($this->datasEmissao[0])) $this->datasEmissao[] = $notafiscal["DTENTRSAID"];
            
            $this->datasEmissao[] = $notafiscal["DTENTRSAID"];
            $this->DataEmissaoRPS = $notafiscal["DTENTRSAID"] . "T" . $dataAtual->format("H:i:s");

            $this->SeriePrestacao = "99";
            
            $this->CidadeTomador = $notafiscal["CDMUNICIBGECLIE"];
            //$this->CidadeTomador = empty($notafiscal["CDMUNICSIAFICLIE"]) ? $notafiscal["CDMUNICIPIOCLIE"] : $notafiscal["CDMUNICSIAFICLIE"];
            
            $this->InscricaoMunicipalTomador = str_replace(array(".","-","/"," "), "", $notafiscal["NRINSMUNICLIE"]);
            if (strtoupper($this->InscricaoMunicipalTomador) == "ISENTO" || strtoupper($this->InscricaoMunicipalTomador) == "ISENTA" || $this->CidadeTomador != '7107') {
                $this->InscricaoMunicipalTomador = "";
            }
            $this->CPFCNPJTomador = str_replace(array("/", "-", ".", " "), "", $notafiscal["NRINSJURCLIE"]);
            $this->RazaoSocialTomador = $notafiscal["NMRAZSOCCLIE"];

            $this->TipoLogradouroTomador = "-";
            $this->LogradouroTomador = substr($notafiscal["DSENDECLIE"], 0, 50);
            $this->NumeroEnderecoTomador = "-";
            $this->ComplementoEnderecoTomador = substr($notafiscal["DSCOMPENDCLI"], 0, 30);
            $this->TipoBairroTomador = "BAIRRO";
            $this->BairroTomador = substr($notafiscal["NMBAIRCLIE"], 0, 50);
            $this->CidadeTomadorDescricao = substr($notafiscal["NMMUNICIPIOCLIE"], 0, 50);
            $this->Uf = $notafiscal["SGESTADOCLIE"];
            
            $this->CEPTomador = str_replace(array(".", "-", " "), "", $notafiscal["NRCEPCLIE"]);
            $this->EmailTomador = empty($notafiscal["DSEMAILCLIENFE"]) ? "-" : substr($notafiscal["DSEMAILCLIENFE"], 0, 60);
            $itemProduto = Util::getItemProduto($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
			$cdarvprod = $itemProduto[0]["CDARVPROD"];
			
            $codTribMuni = Util::getCodTribMuni($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $cdarvprod, $this->environment->getNrOrg());
           
            //VALIDACAO
            if (!is_array($codTribMuni) || empty($codTribMuni["CDCLASSERVPADR"])) {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Código da tributação da atividade deve ser informado."
                );
            }
            else {
                $this->CodigoAtividade = $codTribMuni["CDCLASSERVPADR"];//631190001
            }
            
            $this->AliquotaAtividade = 0.0;
            $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, "31", $this->environment->getNrOrg());
            if (!empty($impostoItem) && !empty($impostoItem["VRALIQITEM"])) {
            	((int)$impostoItem["VRALIQITEM"] > 0)? ($valorAliquota = (float)$impostoItem["VRALIQITEM"] / 100) : ($valorAliquota = (float)$impostoItem["VRALIQITEM"]); // Na tabela tem aliquotas cadastradas em forma de porcentagem e outras em forma de numero inteiro
            	$this->AliquotaAtividade = Util::formatarNumero($valorAliquota, 3);
            }
            
            $this->TipoRecolhimento = "A";
            $this->TipoRecolhimentoAss = "N";
            // $this->MunicipioPrestacao = '7107'; //$notafiscal["CDMUNISERVPRES"]; // Estava sendo usado estático só para SP mas pode variar
            
            $this->MunicipioPrestacao = $notafiscal["CDMUNICIBGESERVPRES"]; // Código do municipio onde está sendo prestado o serviço
            $this->MunicipioPrestacaoDescricao = $notafiscal["NMMUNISERVPRES"] ;
            $estadoPrestacao = $notafiscal["SGESTADOPRESSERV"]; // estado no qual o serviço está sendo prestado
            
            // Caso a nota tenha deduções altera a operação para B
            if(((float)$notafiscal["VRNOTAFISC"] - (float)$notafiscal["VRPRODUTOS"]) > 0) {
                $this->Operacao  = "B";
            }
            else $this->Operacao = "A";
            
            if($this->MunicipioPrestacao != '3550308') { // Valida se o estado de prestacao é fora ou dentro de sp para mudar a tributação
                $this->Tributacao = 'F';
            } 
            else {
                $this->Tributacao = "T";
            }

            $this->ValorPIS = 0.0;
            $this->ValorCOFINS = 0.0;
            $this->ValorINSS = 0.0;
            $this->ValorIR = 0.0;
            $this->ValorCSLL = 0.0;
            $this->IssRetido = "false";
            
            $this->AliquotaPIS = 0.0;
			$this->AliquotaCOFINS = 0.0;
			$this->AliquotaINSS = 0.0;
			$this->AliquotaIR = 0.0;
			$this->AliquotaCSLL = 0.0;

			$impostos = Util::getImpostos($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

            foreach ($impostos as $imp) {
                if (in_array($imp["CDIMPOSTONFE"], array("50", "55", "56")) && $imp["IDTRATIMPO"] == "R") {
                	if ((float)$imp["VRIMPOSTOIT"] > 0) {
                        $this->ValorPIS = Util::formatarNumero($imp["VALORIMPOSTO"]);
                        $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, $imp["CDIMPOSTONFE"], $this->environment->getNrOrg());
			            if (!empty($impostoItem) && !empty($impostoItem["VRALIQITEM"])) {
                            $this->AliquotaPIS = (float)$impostoItem["VRALIQITEM"];
			            }
                	}
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("60", "65", "66")) && $imp["IDTRATIMPO"] == "R") {
                	if ((float)$imp["VRIMPOSTOIT"] > 0) {
	                    $this->ValorCOFINS = Util::formatarNumero($imp["VALORIMPOSTO"]);
                        $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, $imp["CDIMPOSTONFE"], $this->environment->getNrOrg());
			            if (!empty($impostoItem) && !empty($impostoItem["VRALIQITEM"])) {
			            	$this->AliquotaCOFINS = (float)$impostoItem["VRALIQITEM"];
			            }
                	}
                }
                else if ($imp["CDIMPOSTONFE"] == "42" && $imp["IDTRATIMPO"] == "R") {
                	if ((float)$imp["VRIMPOSTOIT"] > 0) {
	                    $this->ValorINSS = Util::formatarNumero($imp["VALORIMPOSTO"]);
                        $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, $imp["CDIMPOSTONFE"], $this->environment->getNrOrg());
			            if (!empty($impostoItem) && !empty($impostoItem["VRALIQITEM"])) {
			            	$this->AliquotaINSS = (float)$impostoItem["VRALIQITEM"];
			            }
                	}
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("22", "25")) && $imp["IDTRATIMPO"] == "R") {
                	if ((float)$imp["VRIMPOSTOIT"] > 0) {
	                    $this->ValorIR = Util::formatarNumero($imp["VALORIMPOSTO"]);
			            $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, $imp["CDIMPOSTONFE"], $this->environment->getNrOrg());
			            if (!empty($impostoItem) && !empty($impostoItem["VRALIQITEM"])) {
			            	$this->AliquotaIR = (float)$impostoItem["VRALIQITEM"];
			            }
                	}
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("70", "75", "76")) && $imp["IDTRATIMPO"] == "R") {
                	if ((float)$imp["VRIMPOSTOIT"] > 0) {
	                    $this->ValorCSLL = Util::formatarNumero($imp["VALORIMPOSTO"]);
			            $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, $imp["CDIMPOSTONFE"], $this->environment->getNrOrg());
			            if (!empty($impostoItem) && !empty($impostoItem["VRALIQITEM"])) {
			            	$this->AliquotaCSLL = (float)$impostoItem["VRALIQITEM"];
			            }
                	}
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("31", "35", "36"))) {
                    if ($imp["IDTRATIMPO"] == "R") {
                        $this->IssRetido = "true";
                        //$this->ValorIssRetido = (float)$imp["VALORIMPOSTO"];
                        //$valorLiquido -= $this->ValorIssRetido;
                    }
                    else {
                       $this->BaseCalculo = $imp["BASECALCULO"];
                    }
                } /* Trecho do código não utilizado
                else if (in_array($imp["CDIMPOSTONFE"], array("30", "31", "32", "35", "36"))) {
                    if ($imp["IDTRATIMPO"] == "R") {
                        $this->TipoRecolhimento = "R";
                        $this->TipoRecolhimentoAss = "S";
                    }
                } */
            }
        
            // o valor aproximado do Total dos Tributos 
            $this->CargaTributaria =  $this->ValorPIS +  $this->ValorCOFINS +  $this->ValorINSS +  $this->ValorIR +  $this->ValorCSLL; 
            $this->CargaTributariaPorcentagem =  $this->CargaTributaria;

            if (count($impostos) == 0) {
                $erros[] = array(
                    "TIPODEERRO" => "5",
                    "CODIGO" => $nota["NRLANCTONF"],
                    "DESCRICAO" => "Impostos da nota devem ser informados."
                );
            }
            $descricaoProd   = "";
            $descricaoRps    = "";
            $dadosAdicionais = Util::getDadosAdicionais($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
    
            if(isset($dadosAdicionais[0])) {
                foreach($dadosAdicionais as $dadoAdicional) {
                    $descricaoRps .= $dadoAdicional["DSDADOADIC"]."\r\n";
                }
            }
            /* Agora a rotina deixa transmitir a nota mesmo se não houver dados adicionais (retirar essa parte comentada(else) se no futuro garantir que realmente não precisa */
    //         else {
    //             foreach ($itemProduto as $linha) {
    //                 $descricaoProd .= $linha["DSPRODEMIS"] . "\r\n";
    //             }
    // 			$erros[] = array(
    //                 "TIPODEERRO" => "5",
    //                 "CODIGO" => $nota["NRLANCTONF"],
    //                 "DESCRICAO" => "Dados Adicionais devem ser informados."
    //             );
    //             return $erros;
    //         }

            $itemEncargo = Util::getItemEncargo($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            $descricaoEncargo = "";
            foreach ($itemEncargo as $item) {
                $descricaoEncargo .= $item["DESCRICAO"]."\n";
            }

            $descricaoDadosAdic = "";
            $dadosAdicionais = Util::getDadosAdicionais($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            foreach ($dadosAdicionais as $item) {
                $descricaoDadosAdic .= $item["DSDADOADIC"];
            }

            $this->DescricaoRPS = substr(trim($descricaoRps), 0, 1500);// . $descricaoEncargo . $descricaoDadosAdic);
     
            $this->DDDPrestador = "";
			$this->TelefonePrestador = "";
            $telefone = str_replace(array("(", ")", "-", " "), "", $notafiscal["NRTELEEMIT"]);
            if (strlen($telefone) == 11) {
            	$this->DDDPrestador = substr($telefone, 0, 3);
            	$this->TelefonePrestador = substr($telefone, 3);
            }
            else if (strlen($telefone) == 10) {
            	$this->DDDPrestador = substr($telefone, 0, 2);
            	$this->TelefonePrestador = substr($telefone, 2);
            }
            $this->DDDTomador = "";
			$this->TelefoneTomador = "";
            $telefone = str_replace(array("(", ")", "-", " "), "", $notafiscal["NRTELECLIE"]);
            if (strlen($telefone) == 11) {
            	$this->DDDTomador = substr($telefone, 0, 3);
            	$this->TelefoneTomador = substr($telefone, 3);
            }
            else if (strlen($telefone) == 10) {
            	$this->DDDTomador = substr($telefone, 0, 2);
            	$this->TelefoneTomador = substr($telefone, 2);
            }

            $this->itensRps = array();
            
			foreach ($itemProduto as $item) {
				// $discriminacaoServico = substr(trim($item["DSPRODEMIS"]), 0, 80); // Estava sendo usado porém cortava para grandes descrições
				$discriminacaoServico = trim($item["DSPRODEMIS"]);
				$discriminacaoServico = $discriminacaoServico != "" ?  $discriminacaoServico :"-";
				$quantidade = (float)$item["QTITEMPROD"];
                $valorUnitario = Util::formatarNumero($item["VRUNITITEM"]);
                $valorTotal = Util::formatarNumero($item["VRTOTAITEM"]);
                $this->ValorIssRetido = ($valorTotal * $this->AliquotaAtividade);
                $IssFormatoDecimal = floor($this->ValorIssRetido*100) / 100; // 2 casas dps da virgula sem arredondar para poder bater com o valor ISQN calculado pela prefeitura
                $this->CargaTributaria += $IssFormatoDecimal;
      
                $impostoItem = Util::getImpostosItem($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg(), "31", $item["NRSEQITPROD"]);
                
                $tributavel = 'S';
                if(isset($impostoItem[0])) {
                    $tributavel = ($impostoItem[0]["CDFISCALIT"] == "1") ? "S" : "N";
                }
               else {
                    $dsErro = "O produto ". $item["DSPRODEMIS"]. " não possui impostos relacionados.";
                    $erros[] = array(
                        "TIPODEERRO" => "5",
                        "NRLANCTONF" => $notafiscal["NRLANCTONF"],
	                    "DESCRICAO"  => $dsErro
                    );
                    return $erros;
                }
				
				
		    $valorServico =  (string) round($notafiscal["VRPRODUTOS"], 2);
            $verificaDecimal = explode(".", $valorServico);
            if(isset($verificaDecimal[1])) {
                if(strlen($verificaDecimal[1]) == 1) {
                    $valorServico.='0';
                }
            }
            else {
                $valorServico.='.00';
            }
            
				$this->itensRps[] = array(
					"DiscriminacaoServico" => "$discriminacaoServico / $descricaoRps", // Concatena os dados adicionais para xml
					"Quantidade" 		   => $quantidade,
					"ValorUnitario" 	   => $valorUnitario,
					"ValorTotal" 		   => $valorTotal,
					"Tributavel" 		   => $tributavel,
					"valorDeducao"         => $valorDeducao,
					"valorServico"          => $valorServico
				);
			}
            $this->ValorTotalServicos = Util::formatarNumero($this->ValorTotalServicos);
            $this->ValorTotalDeducoes = Util::formatarNumero($this->ValorTotalDeducoes);
            
            $dataAssinatura = date("Ymd", strtotime($notafiscal["DTENTRSAID"]));
            $assinatura = 
                str_pad($this->InscricaoMunicipalPrestador, 8, "0", STR_PAD_LEFT)
                . str_pad($this->SerieRPS, 5, ' ', STR_PAD_RIGHT)
                . str_pad($this->NumeroRPS, 12, "0", STR_PAD_LEFT)
                . $dataAssinatura
                . $this->Tributacao
                . $this->SituacaoRPS; 
                
                if($this->IssRetido === "false"){
                    $assinatura .= "N";
                }else {
                    $assinatura .="S";
                }
                
            $assinatura .= str_pad(
                str_replace(['.', ','], '',$valorServico),
                15,
                '0',
                STR_PAD_LEFT
            )
            . str_pad(
                str_replace(['.', ','], '', $valorDeducao),
                15,
                '0',
                STR_PAD_LEFT
            )
            . str_pad( $this->CodigoAtividade, 5, "0", STR_PAD_LEFT);
                    
                if(strlen($this->CPFCNPJTomador) == 14){
                    $assinatura .= '2'
                     . str_pad($this->CPFCNPJTomador, 14, "0", STR_PAD_LEFT);
                    
                }else if (strlen($this->CPFCNPJTomador) == 11) {
                      $assinatura .= '1'
                     . str_pad($this->CPFCNPJTomador, 14, "0", STR_PAD_LEFT);
                }else if ( strlen($this->CPFCNPJTomador) == 0 ){
                     $assinatura .= '3'
                     .str_pad($this->CPFCNPJTomador, 14, "0", STR_PAD_LEFT);
                    
                }
                
           try{
                $data = file_get_contents($this->linkCertificado);
            }
            catch(\Exception $e) {
                return false;
            }
            if(empty($data)) return false;
            openssl_pkcs12_read($data, $cert, $this->senhaCertificadoDigital);
                
            $this->publicKey = $this->getPublicKey($cert["cert"]);
            $this->privateKeyId = openssl_get_privatekey($cert["pkey"]);

           /* $assinatura = $this->byteStr2byteArray($assinatura);
            $assinatura = implode($assinatura);*/
            
            
            $dados = $assinatura;                                                     
            $assinatura = '';
            // var_dump($this->privateKeyId);die;
            openssl_sign($dados, $assinatura, $this->privateKeyId, OPENSSL_ALGO_SHA1);
            openssl_free_key($this->privateKeyId);
            $signatureValue = base64_encode($assinatura);
        
            //$hashValue = hash("sha1", $assinatura, true);
            //$hashValue = sha1($assinatura);

            $this->Assinatura = $signatureValue;
            
            $this->adicionarRpsEnviarLote();
            
            $this->listaAuxRps[] = array(
                "CDSERIERPS"   => $notafiscal["CDSERIERPS"],
                "NRRPS"        => $this->NumeroRPS,
                "CDEMPLOTERPS" => $notafiscal["CDEMPRESA"],
                "CDINSCESTA"   => $notafiscal["CDINSCESTA"],
                "CDINSCMUNI"   => $notafiscal["CDINSCMUNIEMP"],
                "NRSEQLOTERPS" => $numeroLote,
                "NRINSJURIEST" => $notafiscal["NRINSJURIEST"],
                "CDFILIAL"     => $cdfilial,
                "NRLANCTONF"   => $nrlanctonf
            );
        }
        
        if (count($erros) > 1) {
            $this->limparListaRps();
            return $erros;
        }
        else {
            $this->gerarXmlEnviarLoteRps();
            $xml = $this->getXmlAssinadoEnviarLoteRps();
            return $this->enviarLoteRpsWS($xml, $notafiscal["CDEMPRESA"], $notafiscal["NRINSJURIEST"], $numeroLote);
        }
    }

    private function enviarLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        
        $nomeFuncao = "envioLoteRPSAsync"; // No momento ambiente homologação não funciona, descomentar a condição abaixo para testes futuros
        
        // if ($this->ambiente = 1) {
        //     $nomeFuncao = "envioLoteRPSAsync";
        //   //$nomeFuncao = "envioLoteRPS";
        // }
        // else {
        //     $nomeFuncao = "testeEnvioLoteRPSAsync";
        // }
        
        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        //file_put_contents($this->caminhoXml . $prefixoArquivo."env.xml", $xml);
        
        var_dump(['Envio para Prefeitura' => $xml]); // XML de envio e retorno da prefeitura seráo exibitos no $reponse da requisição
        $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        //file_put_contents($this->caminhoXml . $prefixoArquivo."resp.xml", $resultado);
        var_dump(['Retorno da Prefeitura' => $resultado]);

        if (is_string($resultado)) {
            if (substr($resultado, 0, 1) == "<") {
                $domResult = new \DOMDocument("1.0", "UTF-8");
                $domResult->loadXML($resultado);
                if($nomeFuncao === 'envioLoteRPSAsync' ){
                if ($domResult->getElementsByTagName("Sucesso")->item(0)->nodeValue == "true") {
    
                    $protocolo = $domResult->getElementsByTagName("NumeroProtocolo")->item(0)->nodeValue;
                    foreach ($this->listaAuxRps as $nota) {
                        
                        $nota["NRRPS"] = str_pad($nota["NRRPS"], 15, "0", STR_PAD_LEFT);
                        
                        Util::inserirNumeroRps($this->connection, $nota["NRSEQLOTERPS"], 2, $nota["CDEMPLOTERPS"], $nota["CDSERIERPS"], $nota["NRRPS"], $nota["CDINSCESTA"], $nota["CDINSCMUNI"], $nota["NRINSJURIEST"], null, null, $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);
                        
                        Util::atualizarNotaTransmissao($this->connection, $nota["CDSERIERPS"], $nota["NRRPS"], $nota["CDEMPLOTERPS"], $nota["CDINSCESTA"], $nota["CDINSCMUNI"], $nota["NRSEQLOTERPS"], $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                    }
    
                    Util::atualizarLote($this->connection, $this->listaAuxRps[0]["CDEMPLOTERPS"], $this->listaAuxRps[0]["CDINSCESTA"], $this->listaAuxRps[0]["CDINSCMUNI"], $this->NumeroLote, 2, $protocolo, $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
                    return true;
                }
                else {
                    $erros = array("TIPO" => 1);
                    $nodeList = $domResult->getElementsByTagName("Erro");
                    foreach ($nodeList as $erro) {
                        $nodeIdentificacaoRps = $erro->getElementsByTagName("ChaveRPS")->item(0);
           
                        try {
                            $rpsErro = Util::getChildNodeByName($nodeIdentificacaoRps, "NumeroRPS")->nodeValue;
                            foreach($this->listaAuxRps as $item) {
                                if($item["NRRPS"] == $rpsErro) $nota = $item;
                            } 
                        }
                        catch(\Exception $e) {
                            $rpsErro = '';
                            $nota = array('NRLANCTONF' => '');
                        }
                        $erros[] = array(
                            "TIPODEERRO" =>'3',
                            "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                            "NRLANCTONF" => $nota["NRLANCTONF"],
                            "DESCRICAO" => Util::getChildNodeByName($erro, "Descricao")->nodeValue,
                            "NRRPS"  => $rpsErro,
                            "XML"    => $xml
                        );
                    }
                    return $erros;
                }
            }else{
              return $this->ResponseEnvioLoteSync($domResult , $xml);
            }
            }
            else {
                $erros = array("TIPO" => 2);
                $erros["XML"] = $xml;

                $erros[] = array(
                    "CODIGO" => "",
                    "DESCRICAO" => $resultado
                );
                return $erros;
            }
        }
    
        else {
            return false;
        }
    }

    private function assinaXml(\DOMDocument $domXml, array $tags){
        $local = ($this->caminhoCertificado);
        if($local) {
            $dir = $this->caminhoCertificado . $this->cdempresa . "_" . $this->cdinscmuni;
            if(!is_dir($dir)) return false;
            if((!is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) && (!is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12"))) return false;
            else {
                if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.pfx");
                else if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.p12");
            }
        }
        else {

            try{
                $data = file_get_contents($this->linkCertificado);
            }
            catch(\Exception $e) {
                return false;
            }
            if(empty($data)) return false;
        }
     
        openssl_pkcs12_read($data, $cert, $this->senhaCertificadoDigital);

        if (is_null($cert)) {
            return false;
        }
        else {
            //file_put_contents($dir.DIRECTORY_SEPARATOR."certKey.pem", $cert["pkey"]."\r\n".$cert["cert"]);
            //file_put_contents($dir.DIRECTORY_SEPARATOR."priKey.pem", $cert["pkey"]);
            //file_put_contents($dir.DIRECTORY_SEPARATOR."pubKey.pem", $cert["cert"]);

            $this->publicKey = $this->getPublicKey($cert["cert"]);
            $this->privateKeyId = openssl_get_privatekey($cert["pkey"]);

            foreach ($tags as $tag) {
                $domXml = $this->assinaXmlAux($domXml, $tag);
            }

            openssl_free_key($this->privateKeyId);
            return $domXml;
        }

    }
    
    private function assinaXmlAux(\DOMDocument $domXml, $tag) {
        $nodes = $domXml->getElementsByTagName($tag);

        foreach ($nodes as $node) {
            $root = $node; 
            $id = trim($node->getAttribute("Id"));
            $dados = $node->C14N(false, false, null, null);
            $hashValue = hash("sha1", $dados, true);
            $digValue = base64_encode($hashValue);
            $signature = $domXml->createElement("Signature");
            $root->appendChild($signature);
            $signature->setAttribute("xmlns", "http://www.w3.org/2000/09/xmldsig#");
        
            $signedInfo = $domXml->createElement("SignedInfo");
            $signature->appendChild($signedInfo);
        
            $novaTag = $domXml->createElement("CanonicalizationMethod");
            $signedInfo->appendChild($novaTag);
            $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");
        
            $novaTag = $domXml->createElement("SignatureMethod");
            $signedInfo->appendChild($novaTag);
            $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#rsa-sha1");
        
            $reference = $domXml->createElement("Reference");
            $signedInfo->appendChild($reference);
            $reference->setAttribute("URI", "");
        
            $transforms = $domXml->createElement("Transforms");
            $reference->appendChild($transforms);
        
            $novaTag = $domXml->createElement("Transform");
            $transforms->appendChild($novaTag);
            $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#enveloped-signature");
        
            $novaTag = $domXml->createElement("Transform");
            $transforms->appendChild($novaTag);
            $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");
        
            $novaTag = $domXml->createElement("DigestMethod");
            $reference->appendChild($novaTag);
            $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#sha1");
        
            $novaTag = $domXml->createElement("DigestValue", $digValue);
            $reference->appendChild($novaTag);
        
            $dados = $signedInfo->C14N(false, false, null, null);
            
        
            $assinatura = "";
            openssl_sign($dados, $assinatura, $this->privateKeyId);
            $signatureValue = base64_encode($assinatura);
            $novaTag = $domXml->createElement("SignatureValue", $signatureValue);
            $signature->appendChild($novaTag);
        
            $keyInfo = $domXml->createElement("KeyInfo");
            $signature->appendChild($keyInfo);
        
            $X509Data = $domXml->createElement("X509Data");
            $keyInfo->appendChild($X509Data);
        
            $novaTag = $domXml->createElement("X509Certificate", $this->publicKey);
            $X509Data->appendChild($novaTag);
        }
        
        return $domXml;
    }

    private function getPublicKey($key){
        $data = "";
        $arCert = explode("\n", $key);
        foreach ($arCert as $curData) {
            //remove a tag de inicio e fim do certificado
            if (strncmp($curData, "-----BEGIN CERTIFICATE", 22) != 0 &&
                strncmp($curData, "-----END CERTIFICATE", 20) != 0 ) {
                //carrega o resultado numa string
                $data .= trim($curData);
            }
        }
        return $data;
    }
    
    private function getXmlAssinadoConsultaLoteRps($cdemploterps, $cdinscmuni ){
        $this->xmlConsultarLoteRps = str_replace("&", "&amp;", $this->xmlConsultarLoteRps);
        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($this->xmlConsultarLoteRps, LIBXML_NOBLANKS);
        $domXml->formatOutput = false;
        $xmlTemp = $domXml->saveXML();
        
        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xmlTemp);
        $domXml->encoding = "UTF-8";
        $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $cdemploterps, $cdinscmuni, $this->environment->getNrOrg());


        $domXml = $this->assinaXml($domXml, array("PedidoConsultaLote"));
        if ($domXml === false) {
            throw new \Exception("Certificado digital não encontrado!");
        }
        else {
           return $domXml->saveXML($domXml->documentElement);
        }
        
        
    }
    
    private function consultaSituacaoLote($inscricao , $protocolo, $lote) {

        $xml = $this->gerarXmlConsultarSituacaoLote($inscricao, $protocolo);
        //$xml = $this->getXmlAssinadoConsultaSituacaoLote($xml);
        
        $resultado = $this->consultarSituaçaoLoteWS($xml);
          if (is_string($resultado)) {
            $domResult = new \DOMDocument("1.0", "UTF-8");
            $domResult->loadXML($resultado);
            $sucesso = $domResult->getElementsByTagName("Sucesso");
            if($sucesso->length > 0) {
                if ($domResult->getElementsByTagName("Sucesso")->item(0)->nodeValue == "true") {
                    $situacao = $domResult->getElementsByTagName("Situacao")->item(0)->nodeValue;

                    if($situacao == 1) {
                        $chave = "NUMERORPS".$lote["CDEMPLOTERPS"].$lote["NRINSJURIEST"].$lote["CDSERIERPS"];
                        //$nrRps = $this->newCode->getCodeByTableName($chave, $this->environment->getNrOrg(), 1, 15);
                        //$qtdRps = $lote["QTDERPS"];
                        // $nrRps = $nrRps - ($qtdRps + 1);
                        
                        // Util::atualizarNovoCodigo($this->connection, $lote["CDEMPLOTERPS"], $lote["NRINSJURIEST"], $lote["CDSERIERPS"],  $nrRps, $this->environment->getNrOrg());
                        
                        Util::atualizarSituacaoNumeroRpsPeloLote($this->connection, $lote["CDSERIERPS"], $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                        
                        Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
        
                        $erros = array();
                        $nodeList = $domResult->getElementsByTagName("Erro");
                        foreach ($nodeList as $erro) {
                            $erros[] = array(
                                "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                                "DESCRICAO" => Util::getChildNodeByName($erro, "Descricao")->nodeValue
                            );
                        }
                        return $erros;
                    }
                    else if($situacao == 0 || $situacao == 2) {
                        $erros[] = array(
                            // "CODIGO" => $codigoErro,  // codigoErro não está sendo instanciado
                            "DESCRICAO" => "Lote não processado. Tente novamente mais tarde!",
                            "RPS"    => $numeroRps
                        );
                    }

                    return $domResult->getElementsByTagName("NumeroLote")->item(0)->nodeValue;
                }
                else {
                    $chave = "NUMERORPS".$lote["CDEMPLOTERPS"].$lote["NRINSJURIEST"].$lote["CDSERIERPS"];
                    //$nrRps = $this->newCode->getCodeByTableName($chave, $this->environment->getNrOrg(), 1, 15);
                    //$qtdRps = $lote["QTDERPS"];
                    //$nrRps = $nrRps - ($qtdRps + 1);
                    
                    Util::atualizarNovoCodigo($this->connection, $lote["CDEMPLOTERPS"], $lote["NRINSJURIEST"], $lote["CDSERIERPS"],  $nrRps, $this->environment->getNrOrg());
                    
                    Util::atualizarSituacaoNumeroRpsPeloLote($this->connection, $lote["CDSERIERPS"], $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                    
                    Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
                    $erros = array();
                    $nodeList = $domResult->getElementsByTagName("Erro");
                    foreach ($nodeList as $erro) {
                        $erros[] = array(
                            "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                            "DESCRICAO" => Util::getChildNodeByName($erro, "Descricao")->nodeValue
                        );
                    }
                    return $erros;
                }
            }

        }
        else {
            return false;
        }
        
        
    }

    private function gerarXmlConsultarSituacaoLote($inscricao, $protocolo){
        $xmlConsultarSituacaoLote = '<PedidoConsultaSituacaoLote xmlns="http://www.prefeitura.sp.gov.br/nfe">'
            .'<CPFCNPJRemetente xmlns="">';
              	if (strlen($inscricao) == 11) {
    					$xmlConsultarSituacaoLote .='<CPF>'.$inscricao.'<CPF>';
    				}
    				else {
    					$xmlConsultarSituacaoLote .= '<CNPJ>'.$inscricao.'</CNPJ>';
    				}
	            $xmlConsultarSituacaoLote .='</CPFCNPJRemetente>'
            .'<NumeroProtocolo xmlns="">'.$protocolo.'</NumeroProtocolo>'
         .'</PedidoConsultaSituacaoLote>';
         
         return $xmlConsultarSituacaoLote ; 
        
    }
    
    private function consultarSituaçaoLoteWS($xml) {
        $nomeFuncao = "consultaSituacaoLote";
        $retorno = $this->enviarParaWebservice($nomeFuncao, $xml);
        return $retorno;
    }
    
    private function getXmlAssinadoConsultaSituacaoLote($xml) {
        $xml= str_replace("&", "&amp;", $xml);
        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xml, LIBXML_NOBLANKS);
        $domXml->formatOutput = false;
        $xmlTemp = $domXml->saveXML();
        
        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xmlTemp);
        $domXml->encoding = "UTF-8";
        
        $domXml = $this->assinaXml($domXml, array("PedidoConsultaSituacaoLote"));
        if ($domXml === false) {
            throw new \Exception("Certificado digital não encontrado!");
        }
        else {
           return $domXml->saveXML($domXml->documentElement);
        }
    }
    
    private function ResponseEnvioLoteSync($domResult , $xml){
            if ($domResult->getElementsByTagName("Sucesso")->item(0)->nodeValue == "true") {
            $protocolo = Util::getChildNodeByName($nodeList->item($i), "NumeroLote")->nodeValue;
            
            $nodeList = $domResult->getElementsByTagName("ChaveNFe");
                    for ($i = 0; $i < $nodeList->length; $i++) {
                        $this->listaAuxRps[$i]["NRNOTAFISCSE"] = Util::getChildNodeByName($nodeList->item($i), "NumeroNFe")->nodeValue;
                        $this->listaAuxRps[$i]["CDVERIFNFSE"] = Util::getChildNodeByName($nodeList->item($i), "CodigoVerificacao")->nodeValue;
                    }
        
                foreach ($this->listaAuxRps as $item) {
                    $numeroRPS = str_pad($item["NRRPS"], 15, "0", STR_PAD_LEFT);
    
                
                    Util::inserirNumeroRps($this->connection, $item["NRSEQLOTERPS"], 4, $item["CDEMPLOTERPS"], $item["CDSERIERPS"], $numeroRPS, $item["CDINSCESTA"], $item["CDINSCMUNI"], $item["NRINSJURIEST"], $item["NRNOTAFISCSE"], $item["CDVERIFNFSE"], $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);
                
                    Util::atualizarNotaConsulta($this->connection, $item["CDSERIERPS"], $numeroRPS, $item["NRSEQLOTERPS"], $item["NRNOTAFISCSE"], "S", $item["CDFILIAL"], $item["NRLANCTONF"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                }
                
                Util::atualizarLote($this->connection, $this->listaAuxRps[0]["CDEMPLOTERPS"], $this->listaAuxRps[0]["CDINSCESTA"], $this->listaAuxRps[0]["CDINSCMUNI"], $this->listaAuxRps[0]["NRSEQLOTERPS"], 4, $protocolo, $this->environment->getNrOrg(), $this->environment->getCdOperador());
            
            return true;
             
            
        } else {
            $erros = array("TIPO" => 1);
            $nodeList = $domResult->getElementsByTagName("Erro");
            foreach ($nodeList as $erro) {
                $nodeIdentificacaoRps = $erro->getElementsByTagName("ChaveRPS")->item(0);
        
                try {
                    $rpsErro = Util::getChildNodeByName($nodeIdentificacaoRps, "NumeroRPS")->nodeValue;
                    foreach($this->listaAuxRps as $item) {
                        if($item["NRRPS"] == $rpsErro) $nota = $item;
                    } 
                }
                catch(\Exception $e) {
                    $rpsErro = '';
                    $nota = array('NRLANCTONF' => '');
                }
                $erros[] = array(
                    "TIPODEERRO" =>'3',
                    "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                    "NRLANCTONF" => $nota["NRLANCTONF"],
                    "DESCRICAO" => Util::getChildNodeByName($erro, "Descricao")->nodeValue,
                    "NRRPS"  => $rpsErro,
                    "XML"    => $xml
                );
            }
            return $erros;
        }
    }
}